#!/usr/bin/python

###############################################################################
#
# chmgr-port-list - Parses a text file generated by the chassis manager and
#                   generates a data structure containing all of the local
#                   ports assigned to each front panel port.  This data
#                   structure will be used by update-ports.
#
#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation and its affiliates
# (the "Company") and all right, title, and interest in and to the software
# product, including all associated intellectual property rights, are and
# shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.
#
# Description:
#
#
# Syntax:
#      chmgr-port-list [--verbose]
#
#      --verbose    : Verbose mode
#
###############################################################################


import os
import sys
import string
import re
import argparse


#
# Global variables and "constants"
#
ports_list_file	= '/etc/mlx/chmgr_ports_list.txt'


#
# Command line switches
#
verbose	= 0


# Parses the file 'ports_list_file' to determine the number of
# front panel ports in the system.
def chmgr_port_list_num_of_front_panel_ports():
    num_ports = 0
    port_list = []

    try:
        with open(ports_list_file) as file_object:
            #ports_list_file currently has 4 columns
            cols = 4
            for line in file_object:
                line.lstrip()
                if line.startswith('#'):
                    continue

                row = line.split()
                if len(row) == cols:
                    index      = (int) (row[0])
                    slot       = (int) (row[1])
                    label_port = (int) (row[2])
                    local_port = (int) (row[3])
                    if (slot, label_port) not in port_list:
                        port_list.append((slot, label_port))
                        num_ports = num_ports + 1
    except:
        pass

    return num_ports


# Parses the file 'ports_list_file' and constructs
# a switch port configuration data structure.
def chmgr_port_list_switch_port_cfg(num_of_front_panel_ports):
    switch_port_cfg = {}

    try:
        with open(ports_list_file) as file_object:
            #ports_list_file currently has 4 columns
            cols = 4
            for line in file_object:
                line.lstrip()
                if line.startswith('#'):
                    continue

                row = line.split()
                if len(row) == cols:
                    index      = (int) (row[0])
                    slot       = (int) (row[1])
                    label_port = (int) (row[2])
                    local_port = (int) (row[3])

                    if slot == 0:
                        front_panel= '%s' % (label_port)
                    else:
                        front_panel= 'm%sp%s' % (slot, label_port)

                    if front_panel in switch_port_cfg:
                        switch_port_cfg[front_panel].append((label_port,local_port))
                    else:
                        switch_port_cfg[front_panel] = [(label_port,local_port)]
    except:
        pass

    return switch_port_cfg


"""
# 'Main' section of the script

# Decipher command line arguements
parser = argparse.ArgumentParser()
parser.add_argument("--verbose", help="verbose mode", action="store_true")
args = parser.parse_args()
if args.verbose:
    verbose = 1

num_of_front_panel_ports = chmgr_port_list_num_of_front_panel_ports()
if verbose:
    print ("num_of_front_panel_ports = %d\n" % (num_of_front_panel_ports))
if num_of_front_panel_ports == 0:
    sys.exit()

switch_cfg = chmgr_port_list_switch_port_cfg(num_of_front_panel_ports)
if verbose:
    print ("switch_cfg:")
    for i in range(0,num_of_front_panel_ports):
        print (i, switch_cfg[i])
"""
