#
# Copyright (C) 2020-2021 NVIDIA Corporation. ALL RIGHTS RESERVED.
# Copyright 2016, 2017, 2019.  Cumulus Networks, Inc.
#

import os
import subprocess
import cumulus.cpld
import cumulus.sensors

MlxSN2xxx_led_path  = '/run/hw-management/led'

class Mlx2xxxCPLD(cumulus.cpld.SysFS):
    path = '/run/hw-management/thermal'

# System LEDs
class MlxSN2xxxLED(cumulus.sensors.StatusLED_Unit):
    led_path = MlxSN2xxx_led_path
    override_gpio_path = True
    good_lead_color = 'green'
    bad_led_color = 'red'
    green_colormap = {'green':'255', 'red':'0'}
    red_colormap   = {'green':'0', 'red':'255'}
    colormap = {'2550':'green', '0255':'red', '00':'off'}

    def led_update(self, color):
        if color is 'green':
            self.set_gpio_value(self.led_location[0], self.green_colormap[color])
        else:
            self.set_gpio_value(self.led_location[1], self.red_colormap[color])

    def led_read(self):
        green_color = self.get_gpio_value(self.led_location[0])
        red_color = self.get_gpio_value(self.led_location[1])

        if green_color is None:
            green_color = '0'
        if red_color is None:
            red_color = '0'
        return self.name, self.colormap[green_color + red_color]

class MlxSN2xxxStatusLED(MlxSN2xxxLED):
    name = 'System'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6',
               'Fan7', 'Fan8', 'PSU1', 'PSU1Fan1', 'PSU1Temp1',
               'PSU2', 'PSU2Fan1', 'PSU2Temp1', 'Temp1', 'Temp2',
               'Temp3', 'Temp4', 'Temp5', 'Temp6']
    led_location = ['led_status_green', 'led_status_red']

class MlxSN2xxxPsuLED(MlxSN2xxxLED):
    name = 'Psu'
    sources = ['PSU1', 'PSU2']
    led_location = ['led_psu_green', 'led_psu_red']

class MlxSN2xxxFan1LED(MlxSN2xxxLED):
    name = 'Fan Tray 1'
    sources = ['Fan1', 'Fan2']
    led_location = ['led_fan1_green', 'led_fan1_red']

class MlxSN2xxxFan2LED(MlxSN2xxxLED):
    name = 'Fan Tray 2'
    sources = ['Fan3', 'Fan4']
    led_location = ['led_fan2_green', 'led_fan2_red']

class MlxSN2xxxFan3LED(MlxSN2xxxLED):
    name = 'Fan Tray 3'
    sources = ['Fan5', 'Fan6']
    led_location = ['led_fan3_green', 'led_fan3_red']

class MlxSN2xxxFan4LED(MlxSN2xxxLED):
    name = 'Fan Tray 4'
    sources = ['Fan7', 'Fan8']
    led_location = ['led_fan4_green', 'led_fan4_red']

# Base class for the fans
class MlxSN2xxxFan(cumulus.sensors.Fan_Unit):
    fan_max = 20000
    fan_min = 4700
    
    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

class MlxSN2xxxOddFan(MlxSN2xxxFan):
    fan_max = 23500
    fan_min = 5000

# System fan #1
class MlxSN2xxxFan_1(MlxSN2xxxOddFan):
    name = 'Fan1'
    description = 'Fan Tray 1'
    fan_num = name.lower()
    fan_num_status = 'fan1'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #2
class MlxSN2xxxFan_2(MlxSN2xxxFan):
    name = 'Fan2'
    description = 'Fan Tray 1'
    fan_num = name.lower()
    fan_num_status = 'fan1'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #3
class MlxSN2xxxFan_3(MlxSN2xxxOddFan):
    name = 'Fan3'
    description = 'Fan Tray 2'
    fan_num = name.lower()
    fan_num_status = 'fan2'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #4
class MlxSN2xxxFan_4(MlxSN2xxxFan):
    name = 'Fan4'
    description = 'Fan Tray 2'
    fan_num = name.lower()
    fan_num_status = 'fan2'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #5
class MlxSN2xxxFan_5(MlxSN2xxxOddFan):
    name = 'Fan5'
    description = 'Fan Tray 3'
    fan_num = name.lower()
    fan_num_status = 'fan3'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #6
class MlxSN2xxxFan_6(MlxSN2xxxFan):
    name = 'Fan6'
    description = 'Fan Tray 3'
    fan_num = name.lower()
    fan_num_status = 'fan3'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #7
class MlxSN2xxxFan_7(MlxSN2xxxOddFan):
    name = 'Fan7'
    description = 'Fan Tray 4'
    fan_num = name.lower()
    fan_num_status = 'fan4'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #8
class MlxSN2xxxFan_8(MlxSN2xxxFan):
    name = 'Fan8'
    description = 'Fan Tray 4'
    fan_num = name.lower()
    fan_num_status = 'fan4'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# Base class for the PSU Fans
class MlxSN2xxxPSUFan(cumulus.sensors.Fan_Unit):
    fan_max = 20000
    fan_min = 1000

    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'
    mlnx_hw_mgmt_psu_fan = True
    mlnx_hw_mgmt_fan_var = 'fan1_var'
    mlnx_hw_mgmt_fan_status = None
    mlnx_hw_mgmt_fan_fault = None
    fan_attr_alias = 'fan1_input'
    fan_min_attr_alias = 'fan1_min'
    fan_max_attr_alias = 'fan1_max'

# PSU #1 Fan
class MlxSN2xxxPSUFan_1(MlxSN2xxxPSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    mlnx_hw_mgmt_psu_status_nodes = ['psu1_status', 'psu1_pwr_status']
    mlnx_hw_mgmt_fan_speed = 'psu1_fan1_speed_get'
    mlnx_hw_mgmt_fan_min = 'psu1_fan_min'
    mlnx_hw_mgmt_fan_max = 'psu1_fan_max'

# PSU #2 Fan
class MlxSN2xxxPSUFan_2(MlxSN2xxxPSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    mlnx_hw_mgmt_psu_status_nodes = ['psu2_status', 'psu2_pwr_status']
    mlnx_hw_mgmt_fan_speed = 'psu2_fan1_speed_get'
    mlnx_hw_mgmt_fan_min = 'psu2_fan_min'
    mlnx_hw_mgmt_fan_max = 'psu2_fan_max'

# Base class for the Temp Sensors
class MlxSN2xxxTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 75
    temp_max = 105
    temp_crit = 120

    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

# System temp sensor #1, CPU Package
class MlxSN2xxxTemp_1(MlxSN2xxxTemp):
    name = 'Temp1'
    description = 'CPU Package Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_pack'
    mlnx_hw_mgmt_temp_crit = 'cpu_pack_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_pack_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #2
class MlxSN2xxxTemp_2(MlxSN2xxxTemp):
    name = 'Temp2'
    description = 'CPU Core Sensor 0'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core0'
    mlnx_hw_mgmt_temp_crit = 'cpu_core0_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core0_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #3
class MlxSN2xxxTemp_3(MlxSN2xxxTemp):
    name = 'Temp3'
    description = 'CPU Core Sensor 1'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core1'
    mlnx_hw_mgmt_temp_crit = 'cpu_core1_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core1_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #4
class MlxSN2xxxTemp_4(MlxSN2xxxTemp):
    name = 'Temp4'
    temp_fan_min = 40
    temp_max = 60
    temp_crit = 70
    description = 'Port Ambient Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'port_amb'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = None
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #5
class MlxSN2xxxTemp_5(MlxSN2xxxTemp):
    name = 'Temp5'
    temp_fan_min = 40
    temp_max = 60
    temp_crit = 70
    description = 'Main Board Ambient Sensor'
    mlnx_hw_mgmt_temp_val = 'fan_amb'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = None
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #6
class MlxSN2xxxTemp_6(MlxSN2xxxTemp):
    name = 'Temp6'
    temp_fan_min = 80
    temp_max = 105
    temp_crit = 120
    description = 'Asic Temp Sensor'
    mlnx_hw_mgmt_temp_val = 'asic'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = None
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# Base class for the PSU temp sensors
class MlxSN2xxxPSUTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 30
    temp_max = 55
    temp_crit = 105
    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'
    mlnx_hw_mgmt_psu_temp = True
    temp_attr_prefix = 'temp1'
    temp_attr_alias = temp_attr_prefix + '_input'

# PSU1 temp sensor
class MlxSN2xxxPSUTemp_1(MlxSN2xxxPSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    psu = 'PSU1'
    mlnx_hw_mgmt_psu_status_nodes = ['psu1_status', 'psu1_pwr_status']
    mlnx_hw_mgmt_temp_val = 'psu1_temp'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = 'psu1_temp_max'

# PSU2 temp sensor
class MlxSN2xxxPSUTemp_2(MlxSN2xxxPSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    psu = 'PSU2'
    mlnx_hw_mgmt_psu_status_nodes = ['psu2_status', 'psu2_pwr_status']
    mlnx_hw_mgmt_temp_val = 'psu2_temp'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = 'psu2_temp_max'

# Base class for the PSUs
class MlxSN2xxxPSU(cumulus.sensors.PSU_Unit):
    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)

# PSU #1
class MlxSN2xxxPSU1(MlxSN2xxxPSU):
    name = 'PSU1'
    mlnx_hw_mgmt_psu_status = 'psu1_status'
    mlnx_hw_mgmt_psu_pwr_status = 'psu1_pwr_status'

# PSU #2
class MlxSN2xxxPSU2(MlxSN2xxxPSU):
    name = 'PSU2'
    mlnx_hw_mgmt_psu_status = 'psu2_status'
    mlnx_hw_mgmt_psu_pwr_status = 'psu2_pwr_status'

class MlxSN2xxxLEDs():
    units = (
        MlxSN2xxxStatusLED(),
        MlxSN2xxxPsuLED(),
        MlxSN2xxxFan1LED(),
        MlxSN2xxxFan2LED(),
        MlxSN2xxxFan3LED(),
        MlxSN2xxxFan4LED(),
    )

class MlxSN2xxxSensors():

    def __init__(self):
        self.units = (MlxSN2xxxFan_1(),
                      MlxSN2xxxFan_2(),
                      MlxSN2xxxFan_3(),
                      MlxSN2xxxFan_4(),
                      MlxSN2xxxFan_5(),
                      MlxSN2xxxFan_6(),
                      MlxSN2xxxFan_7(),
                      MlxSN2xxxFan_8(),
                      MlxSN2xxxPSUFan_1(),
                      MlxSN2xxxPSUFan_2(),
                      MlxSN2xxxTemp_1(),
                      MlxSN2xxxTemp_2(),
                      MlxSN2xxxTemp_3(),
                      MlxSN2xxxTemp_4(),
                      MlxSN2xxxTemp_5(),
                      MlxSN2xxxTemp_6(),
                      MlxSN2xxxPSUTemp_1(),
                      MlxSN2xxxPSUTemp_2(),
                      MlxSN2xxxPSU1(),
                      MlxSN2xxxPSU2(),
                      )
