#
# Copyright (C) 2020-2021 NVIDIA Corporation. ALL RIGHTS RESERVED.
# Copyright 2020.  Cumulus Networks, Inc.
#

import os
import subprocess
import cumulus.cpld
import cumulus.sensors

MlxSN4xxx_led_path  = '/run/hw-management/led'

class Mlx4xxxCPLD(cumulus.cpld.SysFS):
    path = '/run/hw-management/thermal'

# System LEDs
class MlxSN4xxxLED(cumulus.sensors.StatusLED_Unit):
    led_path = MlxSN4xxx_led_path
    override_gpio_path = True
    good_led_color = 'green'
    bad_led_color = 'orange'
    green_colormap = {'green':'255', 'orange':'0'}
    orange_colormap   = {'green':'0', 'orange':'255'}
    colormap = {'2550':'green', '0255':'orange', '00':'off', '255255': 'orange'}

    def led_update(self, color):
        if color is 'green':
            self.set_gpio_value(self.led_location[0], self.green_colormap[color])
        else:
            self.set_gpio_value(self.led_location[1], self.orange_colormap[color])

    def led_read(self):
        green_color = self.get_gpio_value(self.led_location[0])
        orange_color = self.get_gpio_value(self.led_location[1])

        if green_color is None:
            green_color = '0'
        if orange_color is None:
            orange_color = '0'
        return self.name, self.colormap[green_color + orange_color]

class MlxSN4xxxStatusLED(MlxSN4xxxLED):
    name = 'System'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6',
               'Fan7', 'Fan8', 'PSU1', 'PSU1Fan1', 'PSU1Temp1',
               'PSU2', 'PSU2Fan1', 'PSU2Temp1', 'Temp1', 'Temp2',
               'Temp3', 'Temp4', 'Temp5', 'Temp6']
    led_location = ['led_status_green', 'led_status_orange']

class MlxSN4xxxPsuLED(MlxSN4xxxLED):
    name = 'Psu'
    sources = ['PSU1', 'PSU2']
    led_location = ['led_psu_green', 'led_psu_orange']

class MlxSN4xxxFan1LED(MlxSN4xxxLED):
    name = 'Fan Tray 1'
    sources = ['Fan1']
    led_location = ['led_fan1_green', 'led_fan1_orange']

class MlxSN4xxxFan2LED(MlxSN4xxxLED):
    name = 'Fan Tray 2'
    sources = ['Fan2']
    led_location = ['led_fan2_green', 'led_fan2_orange']

class MlxSN4xxxFan3LED(MlxSN4xxxLED):
    name = 'Fan Tray 3'
    sources = ['Fan3']
    led_location = ['led_fan3_green', 'led_fan3_orange']

class MlxSN4xxxFan4LED(MlxSN4xxxLED):
    name = 'Fan Tray 4'
    sources = ['Fan4']
    led_location = ['led_fan4_green', 'led_fan4_orange']

class MlxSN4xxxFan5LED(MlxSN4xxxLED):
    name = 'Fan Tray 5'
    sources = ['Fan5']
    led_location = ['led_fan5_green', 'led_fan5_orange']

class MlxSN4xxxFan6LED(MlxSN4xxxLED):
    name = 'Fan Tray 6'
    sources = ['Fan6']
    led_location = ['led_fan6_green', 'led_fan6_orange']

# Base class for the fans
class MlxSN4xxxFan(cumulus.sensors.Fan_Unit):
    fan_max = 25000
    fan_min = 4500

    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

# System fan #1
class MlxSN4xxxFan_1(MlxSN4xxxFan):
    name = 'Fan1'
    description = 'Fan Tray 1 Front'
    fan_num = name.lower()
    fan_num_status = 'fan1'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #2
class MlxSN4xxxFan_2(MlxSN4xxxFan):
    name = 'Fan2'
    description = 'Fan Tray 1 Rear'
    fan_num = name.lower()
    fan_num_status = 'fan1'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #3
class MlxSN4xxxFan_3(MlxSN4xxxFan):
    name = 'Fan3'
    description = 'Fan Tray 2 Front'
    fan_num = name.lower()
    fan_num_status = 'fan2'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #4
class MlxSN4xxxFan_4(MlxSN4xxxFan):
    name = 'Fan4'
    description = 'Fan Tray 2 Rear'
    fan_num = name.lower()
    fan_num_status = 'fan2'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #5
class MlxSN4xxxFan_5(MlxSN4xxxFan):
    name = 'Fan5'
    description = 'Fan Tray 3 Front'
    fan_num = name.lower()
    fan_num_status = 'fan3'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #6
class MlxSN4xxxFan_6(MlxSN4xxxFan):
    name = 'Fan6'
    description = 'Fan Tray 3 Rear'
    fan_num = name.lower()
    fan_num_status = 'fan3'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #7
class MlxSN4xxxFan_7(MlxSN4xxxFan):
    name = 'Fan7'
    description = 'Fan Tray 4 Front'
    fan_num = name.lower()
    fan_num_status = 'fan4'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #8
class MlxSN4xxxFan_8(MlxSN4xxxFan):
    name = 'Fan8'
    description = 'Fan Tray 4 Rear'
    fan_num = name.lower()
    fan_num_status = 'fan4'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #9
class MlxSN4xxxFan_9(MlxSN4xxxFan):
    name = 'Fan9'
    description = 'Fan Tray 5 Front'
    fan_num = name.lower()
    fan_num_status = 'fan5'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #10
class MlxSN4xxxFan_10(MlxSN4xxxFan):
    name = 'Fan10'   
    description = 'Fan Tray 5 Rear'
    fan_num = name.lower()
    fan_num_status = 'fan5'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #11
class MlxSN4xxxFan_11(MlxSN4xxxFan):
    name = 'Fan11'
    description = 'Fan Tray 6 Front'
    fan_num = name.lower()
    fan_num_status = 'fan6'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #12
class MlxSN4xxxFan_12(MlxSN4xxxFan):
    name = 'Fan12'
    description = 'Fan Tray 6 Rear'
    fan_num = name.lower()
    fan_num_status = 'fan6'
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num_status + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# Base class for the PSU Fans
class MlxSN4xxxPSUFan(cumulus.sensors.Fan_Unit):
    fan_max = 25000
    fan_min = 1000

    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'
    mlnx_hw_mgmt_psu_fan = True
    mlnx_hw_mgmt_fan_var = 'fan1_var'
    mlnx_hw_mgmt_fan_status = None
    mlnx_hw_mgmt_fan_fault = None
    fan_attr_alias = 'fan1_input'
    fan_min_attr_alias = 'fan1_min'
    fan_max_attr_alias = 'fan1_max'

# PSU #1 Fan
class MlxSN4xxxPSUFan_1(MlxSN4xxxPSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    mlnx_hw_mgmt_psu_status_nodes = ['psu1_status', 'psu1_pwr_status']
    mlnx_hw_mgmt_fan_speed = 'psu1_fan1_speed_get'
    mlnx_hw_mgmt_fan_min = 'psu1_fan_min'
    mlnx_hw_mgmt_fan_max = 'psu1_fan_max'

# PSU #2 Fan
class MlxSN4xxxPSUFan_2(MlxSN4xxxPSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    mlnx_hw_mgmt_psu_status_nodes = ['psu2_status', 'psu2_pwr_status']
    mlnx_hw_mgmt_fan_speed = 'psu2_fan1_speed_get'
    mlnx_hw_mgmt_fan_min = 'psu2_fan_min'
    mlnx_hw_mgmt_fan_max = 'psu2_fan_max'

# Base class for the Temp Sensors
class MlxSN4xxxTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 75
    temp_max = 95
    temp_crit = 115

    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

# System temp sensor #1, CPU Package
class MlxSN4xxxTemp_1(MlxSN4xxxTemp):
    name = 'Temp1'
    description = 'CPU Package Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_pack'
    mlnx_hw_mgmt_temp_crit = 'cpu_pack_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_pack_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #2
class MlxSN4xxxTemp_2(MlxSN4xxxTemp):
    name = 'Temp2'
    description = 'CPU Core Sensor 0'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core0'
    mlnx_hw_mgmt_temp_crit = 'cpu_core0_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core0_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #3
class MlxSN4xxxTemp_3(MlxSN4xxxTemp):
    name = 'Temp3'
    description = 'CPU Core Sensor 1'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core1'
    mlnx_hw_mgmt_temp_crit = 'cpu_core1_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core1_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #4
class MlxSN4xxxTemp_4(MlxSN4xxxTemp):
    name = 'Temp4'
    description = 'CPU Core Sensor 2'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core2'
    mlnx_hw_mgmt_temp_crit = 'cpu_core2_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core2_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
 
# System temp sensor #5
class MlxSN4xxxTemp_5(MlxSN4xxxTemp):
    name = 'Temp5'
    description = 'CPU Core Sensor 3'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core3'
    mlnx_hw_mgmt_temp_crit = 'cpu_core3_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core3_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #6
class MlxSN4xxxTemp_6(MlxSN4xxxTemp):
    name = 'Temp6'
    temp_fan_min = 40
    temp_max = 60
    temp_crit = 70
    description = 'Port Ambient Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'port_amb'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = None
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #7
class MlxSN4xxxTemp_7(MlxSN4xxxTemp):
    name = 'Temp7'
    temp_fan_min = 40
    temp_max = 60
    temp_crit = 70
    description = 'Main Board Ambient Sensor'
    mlnx_hw_mgmt_temp_val = 'fan_amb'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = None
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #8
class MlxSN4xxxTemp_8(MlxSN4xxxTemp):
    name = 'Temp8'
    temp_fan_min = 80
    temp_max = 105
    temp_crit = 120
    description = 'Asic Temp Sensor'
    mlnx_hw_mgmt_temp_val = 'asic'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = None
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# Base class for the PSU temp sensors
class MlxSN4xxxPSUTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 30
    temp_max = 55
    temp_crit = 105

    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'
    mlnx_hw_mgmt_psu_temp = True
    temp_attr_prefix = 'temp1'
    temp_attr_alias = temp_attr_prefix + '_input'

# PSU1 temp sensor
class MlxSN4xxxPSUTemp_1(MlxSN4xxxPSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    psu = 'PSU1'
    mlnx_hw_mgmt_psu_status_nodes = ['psu1_status', 'psu1_pwr_status']
    mlnx_hw_mgmt_temp_val = 'psu1_temp'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = 'psu1_temp_max' 

# PSU2 temp sensor
class MlxSN4xxxPSUTemp_2(MlxSN4xxxPSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    psu = 'PSU2'
    mlnx_hw_mgmt_psu_status_nodes = ['psu2_status', 'psu2_pwr_status']
    mlnx_hw_mgmt_temp_val = 'psu2_temp'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = 'psu2_temp_max' 

# Base class for the PSUs
class MlxSN4xxxPSU(cumulus.sensors.PSU_Unit):
    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)

# PSU #1
class MlxSN4xxxPSU1(MlxSN4xxxPSU):
    name = 'PSU1'
    description = 'Power Supply Unit 1'
    mlnx_hw_mgmt_psu_status = 'psu1_status'
    mlnx_hw_mgmt_psu_pwr_status = 'psu1_pwr_status'

# PSU #2
class MlxSN4xxxPSU2(MlxSN4xxxPSU):
    name = 'PSU2'
    description = 'Power Supply Unit 2'
    mlnx_hw_mgmt_psu_status = 'psu2_status'
    mlnx_hw_mgmt_psu_pwr_status = 'psu2_pwr_status'

# PSU #3
class MlxSN4xxxPSU3(MlxSN4xxxPSU):
    name = 'PSU3'
    description = 'Power Supply Unit 3'
    mlnx_hw_mgmt_psu_status = 'psu3_status'
    mlnx_hw_mgmt_psu_pwr_status = 'psu3_pwr_status'

# PSU #4
class MlxSN4xxxPSU4(MlxSN4xxxPSU):
    name = 'PSU4'
    description = 'Power Supply Unit 4'
    mlnx_hw_mgmt_psu_status = 'psu4_status'
    mlnx_hw_mgmt_psu_pwr_status = 'psu4_pwr_status'

class MlxSN4xxxSensors():

    def __init__(self):
        self.units = (MlxSN4xxxFan_1(),
                      MlxSN4xxxFan_2(),
                      MlxSN4xxxFan_3(),
                      MlxSN4xxxFan_4(),
                      MlxSN4xxxFan_5(),
                      MlxSN4xxxFan_6(),
                      MlxSN4xxxFan_7(),
                      MlxSN4xxxFan_8(),
                      MlxSN4xxxFan_9(),
                      MlxSN4xxxFan_10(),
                      MlxSN4xxxFan_11(),
                      MlxSN4xxxFan_12(),
                      MlxSN4xxxPSUFan_1(),
                      MlxSN4xxxPSUFan_2(),
                      MlxSN4xxxTemp_1(),
                      MlxSN4xxxTemp_2(),
                      MlxSN4xxxTemp_3(),
                      MlxSN4xxxTemp_4(),
                      MlxSN4xxxTemp_5(),
                      MlxSN4xxxTemp_6(),
                      MlxSN4xxxTemp_7(),
                      MlxSN4xxxTemp_8(),
                      MlxSN4xxxPSUTemp_1(),
                      MlxSN4xxxPSUTemp_2(),
                      MlxSN4xxxPSU1(),
                      MlxSN4xxxPSU2(),
                      )
