#
# Copyright (C) 2020-2021 NVIDIA Corporation. ALL RIGHTS RESERVED.
# Copyright 2017.  Cumulus Networks, Inc.
#

import os
import subprocess
import cumulus.cpld
import cumulus.platform
import cumulus.sensors
import cumulus.sfp
import cumulus.platforms.mlnx_SN2xxx
import cumulus.platforms.mlnx_x86_MSN2700

MlxSN2740_led_path = '/run/hw-management/led'

class MlxSN2740CPLD(cumulus.cpld.SysFS):
    path = '/run/hw-management/thermal'

# System LEDs
class MlxSN2740StatusLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4',
               'PSU1', 'PSU1Fan1', 'PSU1Temp1',
               'PSU2', 'PSU2Fan1', 'PSU2Temp1',
               'Temp1', 'Temp2', 'Temp3',
               'Temp4', 'Temp5', 'Temp6', 'Temp7']
    led_path = MlxSN2740_led_path
    led_location = 'led_status'
    bad_led_color = 'red'

class MlxSN2740PsuLED(cumulus.sensors.StatusLED_Unit):
    name = 'Psu'
    led_path = MlxSN2740_led_path
    led_location = 'led_psu'
    sources = ['PSU1', 'PSU2']
    bad_led_color = 'red'

class MlxSN2740FanLED(cumulus.sensors.StatusLED_Unit):
    led_path = MlxSN2740_led_path
    bad_led_color = 'red'

class MlxSN2740Fan1LED(MlxSN2740FanLED):
    name = 'Fan 1'
    sources = ['Fan1']
    led_location = 'led_fan1'

class MlxSN2740Fan2LED(MlxSN2740FanLED):
    name = 'Fan 2'
    sources = ['Fan2']
    led_location = 'led_fan2'

class MlxSN2740Fan3LED(MlxSN2740FanLED):
    name = 'Fan 3'
    sources = ['Fan3']
    led_location = 'led_fan3'

class MlxSN2740Fan4LED(MlxSN2740FanLED):
    name = 'Fan 4'
    sources = ['Fan4']
    led_location = 'led_fan4'

# Base class for the fans
class MlxSN2740Fan(cumulus.sensors.Fan_Unit):
    fan_max = 23500
    fan_min = 4000

    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

class MlxSN2740OddFan(MlxSN2740Fan):
    fan_max = 23000
    fan_min = 4000

# System fan #1
class MlxSN2740Fan_1(MlxSN2740OddFan):
    name = 'Fan1'
    description = 'Fan 1'
    fan_num = name.lower()
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #2
class MlxSN2740Fan_2(MlxSN2740Fan):
    name = 'Fan2'
    description = 'Fan 2'
    fan_num = name.lower()
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #3
class MlxSN2740Fan_3(MlxSN2740OddFan):
    name = 'Fan3'
    description = 'Fan 3'
    fan_num = name.lower()
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #4
class MlxSN2740Fan_4(MlxSN2740Fan):
    name = 'Fan4'
    description = 'Fan 4'
    fan_num = name.lower()
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# Base class for the PSU Fans
class MlxSN2740PSUFan(cumulus.sensors.Fan_Unit):
    fan_max = 20000
    fan_min = 1000

    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'
    mlnx_hw_mgmt_psu_fan = True
    mlnx_hw_mgmt_fan_var = 'fan1_var'
    mlnx_hw_mgmt_fan_status = None
    mlnx_hw_mgmt_fan_fault = None
    fan_attr_alias = 'fan1_input'
    fan_min_attr_alias = 'fan1_min'
    fan_max_attr_alias = 'fan1_max'

# PSU #1 Fan
class MlxSN2740PSUFan_1(MlxSN2740PSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    mlnx_hw_mgmt_psu_status_nodes = ['psu1_status', 'psu1_pwr_status']
    mlnx_hw_mgmt_fan_speed = 'psu1_fan1_speed_get'
    mlnx_hw_mgmt_fan_min = 'psu1_fan_min'
    mlnx_hw_mgmt_fan_max = 'psu1_fan_max'

# PSU #2 Fan
class MlxSN2740PSUFan_2(MlxSN2740PSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    mlnx_hw_mgmt_psu_status_nodes = ['psu2_status', 'psu2_pwr_status']
    mlnx_hw_mgmt_fan_speed = 'psu2_fan1_speed_get'
    mlnx_hw_mgmt_fan_min = 'psu2_fan_min'
    mlnx_hw_mgmt_fan_max = 'psu2_fan_max'

# Base class for the Temp Sensors
class MlxSN2740Temp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 75
    temp_max = 95
    temp_crit = 115

    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

# System temp #1, CPU Package
class MlxSN2740Temp_1(MlxSN2740Temp):
    name = 'Temp1'
    description = 'CPU Core Sensor 0'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core0'
    mlnx_hw_mgmt_temp_crit = 'cpu_core0_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core0_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp #2, CPU Core 0
class MlxSN2740Temp_2(MlxSN2740Temp):
    name = 'Temp2'
    description = 'CPU Core Sensor 1'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core1'
    mlnx_hw_mgmt_temp_crit = 'cpu_core1_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core1_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp #3, CPU Core 2
class MlxSN2740Temp_3(MlxSN2740Temp):
    name = 'Temp3'
    description = 'CPU Core Sensor 2'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core2'
    mlnx_hw_mgmt_temp_crit = 'cpu_core2_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core2_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp #4, CPU Core 3
class MlxSN2740Temp_4(MlxSN2740Temp):
    name = 'Temp4'
    description = 'CPU Core Sensor 3'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core3'
    mlnx_hw_mgmt_temp_crit = 'cpu_core3_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core3_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp #5, Port Ambient Sensor
class MlxSN2740Temp_5(MlxSN2740Temp):
    name = 'Temp5'
    temp_fan_min = 40
    temp_max = 100
    temp_crit = 110
    description = 'Port Ambient Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'port_amb'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = None
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp #6, Main Board Ambient
class MlxSN2740Temp_6(MlxSN2740Temp):
    name = 'Temp6'
    temp_fan_min = 40
    temp_max = 100
    temp_crit = 110
    description = 'Main Board Ambient Sensor'
    mlnx_hw_mgmt_temp_val = 'fan_amb'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = None
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp #7, Asic Temp
class MlxSN2740Temp_7(MlxSN2740Temp):
    name = 'Temp7'
    temp_fan_min = 80
    temp_max = 100
    temp_crit = 120
    description = 'Asic Temp Sensor'
    mlnx_hw_mgmt_temp_val = 'asic'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = None
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# Base class for the PSU temp sensors
class MlxSN2740PSUTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 30
    temp_max = 55
    temp_crit = 105
    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'
    mlnx_hw_mgmt_psu_temp = True
    temp_attr_prefix = 'temp1'
    temp_attr_alias = temp_attr_prefix + '_input'

# PSU1 temp sensor
class MlxSN2740PSUTemp_1(MlxSN2740PSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    psu = 'PSU1'
    mlnx_hw_mgmt_psu_status_nodes = ['psu1_status', 'psu1_pwr_status']
    mlnx_hw_mgmt_temp_val = 'psu1_temp'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = 'psu1_temp_max'

# PSU2 temp sensor
class MlxSN2740PSUTemp_2(MlxSN2740PSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    psu = 'PSU2'
    mlnx_hw_mgmt_psu_status_nodes = ['psu2_status', 'psu2_pwr_status']
    mlnx_hw_mgmt_temp_val = 'psu2_temp'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = 'psu2_temp_max'

# Base class for the PSUs
class MlxSN2740PSU(cumulus.sensors.PSU_Unit):
    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)

# PSU #1
class MlxSN2740PSU1(MlxSN2740PSU):
    name = 'PSU1'
    mlnx_hw_mgmt_psu_status = 'psu1_status'
    mlnx_hw_mgmt_psu_pwr_status = 'psu1_pwr_status'

# PSU #2
class MlxSN2740PSU2(MlxSN2740PSU):
    name = 'PSU2'
    mlnx_hw_mgmt_psu_status = 'psu2_status'
    mlnx_hw_mgmt_psu_pwr_status = 'psu2_pwr_status'

class MlxSN2740LEDs():
    units = (
        MlxSN2740StatusLED(),
        MlxSN2740PsuLED(),
        MlxSN2740Fan1LED(),
        MlxSN2740Fan2LED(),
        MlxSN2740Fan3LED(),
        MlxSN2740Fan4LED(),
    )

class MlxSN2740Sensors():

    def __init__(self):
        self.units = (
                 MlxSN2740Fan_1(),
                 MlxSN2740Fan_2(),
                 MlxSN2740Fan_3(),
                 MlxSN2740Fan_4(),
                 MlxSN2740PSUFan_1(),
                 MlxSN2740PSUFan_2(),
                 MlxSN2740Temp_1(),
                 MlxSN2740Temp_2(),
                 MlxSN2740Temp_3(),
                 MlxSN2740Temp_4(),
                 MlxSN2740Temp_5(),
                 MlxSN2740Temp_6(),
                 MlxSN2740Temp_7(),
                 MlxSN2740PSUTemp_1(),
                 MlxSN2740PSUTemp_2(),
                 MlxSN2740PSU1(),
                 MlxSN2740PSU2(),
        )

class MlxSN2740(cumulus.platform.Platform):
    name = 'mlnx,msn2740'
    chip = cumulus.platform.SpectrumChip()
    switch = cumulus.platforms.mlnx_x86_MSN2700.MlxSN2700Switch(chip)
    cpld = MlxSN2740CPLD()
    sensors = MlxSN2740Sensors()
    pwms = None
    statusleds = MlxSN2740LEDs()

Platform = MlxSN2740
