#
# Copyright (C) 2020-2021 NVIDIA Corporation. ALL RIGHTS RESERVED.
# Copyright 2020.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.platforms.mlnx_SN3xxx

class MlxSN3420SFP28(cumulus.platform.SpectrumTwo25GPort):
    _set_led = '''
;;; MlxSN3420SFP28
'''
    def __init__(self, label, adj_label, phy_addr, hw_port):
        hw_port_name = 'xe%d' % hw_port

        cumulus.platform.SpectrumTwo25GPort.__init__(self, label=label,
                        adj_label=adj_label, hw_port=hw_port_name,
                        phy_addr=phy_addr)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_port, 'sfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls


class MlxSN3420QSFP28(cumulus.platform.SpectrumTwo100GPortRate):
    _set_led = '''
;;; MlxSN3420QSFP28
;;; Two bits per port: yellow(active high), green(active high)
'''

    def __init__(self, label, adj_label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.SpectrumTwo100GPortRate.__init__(
            self,
            label=label,
            adj_label=adj_label,
            hw_ports=tuple(hw_port_names),
            phy_addrs=phy_addrs,
            blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls


class MlxSN3420Switch(cumulus.platform.SpectrumSwitch):

    sfp = MlxSN3420SFP28
    _qsfp = MlxSN3420QSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can
    #    be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        sfp(label="nve",  adj_label=None, phy_addr=0,      hw_port=0),
        sfp(label="1",  adj_label=None, phy_addr=0,      hw_port=105),
        sfp(label="2",  adj_label=None, phy_addr=1,      hw_port=106),
        sfp(label="3",  adj_label=None, phy_addr=2,      hw_port=49),
        sfp(label="4",  adj_label=None, phy_addr=3,      hw_port=113),
        sfp(label="5",  adj_label=None, phy_addr=4,      hw_port=114),
        sfp(label="6",  adj_label=None, phy_addr=5,      hw_port=50),
        sfp(label="7",  adj_label=None, phy_addr=6,      hw_port=115),
        sfp(label="8",  adj_label=None, phy_addr=7,      hw_port=116),
        sfp(label="9",  adj_label=None, phy_addr=8,      hw_port=41),
        sfp(label="10", adj_label=None, phy_addr=9,      hw_port=121),
        sfp(label="11", adj_label=None, phy_addr=10,     hw_port=122),
        sfp(label="12", adj_label=None, phy_addr=11,     hw_port=42),
        sfp(label="13", adj_label=None, phy_addr=12,     hw_port=123),
        sfp(label="14", adj_label=None, phy_addr=13,     hw_port=124),
        sfp(label="15", adj_label=None, phy_addr=14,     hw_port=43),
        sfp(label="16", adj_label=None, phy_addr=15,     hw_port=57),
        sfp(label="17", adj_label=None, phy_addr=16,     hw_port=58),
        sfp(label="18", adj_label=None, phy_addr=17,     hw_port=44),
        sfp(label="19", adj_label=None, phy_addr=18,     hw_port=59),
        sfp(label="20", adj_label=None, phy_addr=19,     hw_port=60),
        sfp(label="21", adj_label=None, phy_addr=20,     hw_port=33),
        sfp(label="22", adj_label=None, phy_addr=21,     hw_port=51),
        sfp(label="23", adj_label=None, phy_addr=22,     hw_port=52),
        sfp(label="24", adj_label=None, phy_addr=23,     hw_port=34),
        sfp(label="25", adj_label=None, phy_addr=24,     hw_port=53),
        sfp(label="26", adj_label=None, phy_addr=25,     hw_port=54),
        sfp(label="27", adj_label=None, phy_addr=26,     hw_port=35),
        sfp(label="28", adj_label=None, phy_addr=27,     hw_port=45),
        sfp(label="29", adj_label=None, phy_addr=28,     hw_port=46),
        sfp(label="30", adj_label=None, phy_addr=29,     hw_port=36),
        sfp(label="31", adj_label=None, phy_addr=30,     hw_port=47),
        sfp(label="32", adj_label=None, phy_addr=31,     hw_port=48),
        sfp(label="33", adj_label=None, phy_addr=32,     hw_port=1),
        sfp(label="34", adj_label=None, phy_addr=33,     hw_port=37),
        sfp(label="35", adj_label=None, phy_addr=34,     hw_port=38),
        sfp(label="36", adj_label=None, phy_addr=35,     hw_port=2),
        sfp(label="37", adj_label=None, phy_addr=36,     hw_port=39),
        sfp(label="38", adj_label=None, phy_addr=37,     hw_port=40),
        sfp(label="39", adj_label=None, phy_addr=38,     hw_port=3),
        sfp(label="40", adj_label=None, phy_addr=39,     hw_port=4),
        sfp(label="41", adj_label=None, phy_addr=40,     hw_port=5),
        sfp(label="42", adj_label=None, phy_addr=41,     hw_port=6),
        sfp(label="43", adj_label=None, phy_addr=42,     hw_port=7),
        sfp(label="44", adj_label=None, phy_addr=43,     hw_port=8),
        sfp(label="45", adj_label=None, phy_addr=44,     hw_port=9),
        sfp(label="46", adj_label=None, phy_addr=45,     hw_port=10),
        sfp(label="47", adj_label=None, phy_addr=46,     hw_port=11),
        sfp(label="48", adj_label=None, phy_addr=47,     hw_port=12),

        _qsfp(label="49", adj_label=None,
              phy_addrs=(48, 48, 48, 48), hw_ports=(17, 18, 19, 20)),
        _qsfp(label="50", adj_label=None,
              phy_addrs=(49, 49, 49, 49), hw_ports=(21, 22, 23, 24)),
        _qsfp(label="51", adj_label=None,
              phy_addrs=(50, 50, 50, 50), hw_ports=(25, 26, 27, 28)),
        _qsfp(label="52", adj_label=None,
              phy_addrs=(51, 51, 51, 51), hw_ports=(29, 30, 31, 32)),
        _qsfp(label="53", adj_label=None,
              phy_addrs=(52, 52, 52, 52), hw_ports=(89, 90, 91, 92)),
        _qsfp(label="54", adj_label=None,
              phy_addrs=(53, 53, 53, 53), hw_ports=(93, 94, 95, 96)),
        _qsfp(label="55", adj_label=None,
              phy_addrs=(54, 54, 54, 54), hw_ports=(81, 82, 83, 84)),
        _qsfp(label="56", adj_label=None,
              phy_addrs=(55, 55, 55, 55), hw_ports=(85, 86, 87, 88)),
        _qsfp(label="57", adj_label=None,
              phy_addrs=(56, 56, 56, 56), hw_ports=(73, 74, 75, 76)),
        _qsfp(label="58", adj_label=None,
              phy_addrs=(57, 57, 57, 57), hw_ports=(77, 78, 79, 80)),
        _qsfp(label="59", adj_label=None,
              phy_addrs=(58, 58, 58, 58), hw_ports=(65, 66, 67, 68)),
        _qsfp(label="60", adj_label=None,
              phy_addrs=(59, 59, 59, 59), hw_ports=(69, 70, 71, 72)),
    )

class MlxSN3420LEDS():
    units = (
        cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxStatusLED(),
        cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxPsuLED(),
        cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan1LED(),
        cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan2LED(),
        cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan3LED(),
        cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan4LED(),
        cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan5LED(),
    )

class MlxSN3420Sensors():

    def __init__(self):
        self.units = (cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan_1(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan_2(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan_3(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan_4(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan_5(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan_6(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan_7(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan_8(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan_9(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxFan_10(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxPSUFan_1(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxPSUFan_2(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxTemp_1(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxTemp_2(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxTemp_3(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxTemp_4(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxTemp_5(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxTemp_6(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxPSUTemp_1(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxPSUTemp_2(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxPSU1(),
                      cumulus.platforms.mlnx_SN3xxx.MlxSN3xxxPSU2(),
                      )

class MlxSN3420(cumulus.platform.Platform):
    name = 'mlnx,msn3420'
    chip = cumulus.platform.SpectrumChip()
    switch = MlxSN3420Switch(chip)
    cpld = cumulus.platforms.mlnx_SN3xxx.Mlx2xxxCPLD()
    sensors = MlxSN3420Sensors()
    pwms = None
    statusleds = MlxSN3420LEDS
    use_rate_api = True


Platform = MlxSN3420
