#
# Copyright (C) 2020-2021 NVIDIA Corporation. ALL RIGHTS RESERVED.
# Copyright 2020-2021.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.platforms.mlnx_SN4xxx

class MlxSN4410Fan1LED(cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan1LED):
    sources = ['Fan1', 'Fan2']

class MlxSN4410Fan2LED(cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan2LED):
    sources = ['Fan3', 'Fan4']

class MlxSN4410Fan3LED(cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan3LED):
    sources = ['Fan5', 'Fan6']

class MlxSN4410Fan4LED(cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan4LED):
    sources = ['Fan7', 'Fan8']

class MlxSN4410Fan5LED(cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan5LED):
    sources = ['Fan9', 'Fan10']

class MlxSN4410Fan6LED(cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan6LED):
    sources = ['Fan11', 'Fan12']

class MlxSN4410QSFP28(cumulus.platform.SpectrumThree100GPort):
    _set_led = '''
;;; MlxSN4410QSFP28
;;; Two bits per port: yellow(active high), green(active high)
'''

    def __init__(self, label, adj_label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.SpectrumThree100GPort.__init__(
            self,
            label=label,
            adj_label=adj_label,
            hw_ports=tuple(hw_port_names),
            phy_addrs=phy_addrs,
            blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls


class MlxSN4410QSFP56(cumulus.platform.SpectrumThree400GPort):
    _set_led = '''
;;; MlxSN4410QSFP56
;;; Two bits per port: yellow(active high), green(active high)
'''

    def __init__(self, label, adj_label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.SpectrumThree400GPort.__init__(
            self,
            label=label,
            adj_label=adj_label,
            hw_ports=tuple(hw_port_names),
            phy_addrs=phy_addrs,
            blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp56')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls


class MlxSN4410Switch(cumulus.platform.SpectrumSwitch):

    _sfp    = None
    _qsfp   = MlxSN4410QSFP28
    _qsfpdd = MlxSN4410QSFP56
 
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can
    #    be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        _qsfp(label="nve", adj_label=None,
              phy_addrs=( 0,  0,  0,  0), hw_ports=(  0,   0)),
        _qsfp(label="1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0), hw_ports=(105, 106, 107, 108)),
        _qsfp(label="2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1), hw_ports=(109, 110, 111, 112)),
        _qsfp(label="3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2), hw_ports=( 97,  98,  99, 100)),
        _qsfp(label="4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3), hw_ports=(101, 102, 103, 104)),
        _qsfp(label="5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4), hw_ports=(121, 122, 123, 124)),
        _qsfp(label="6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5), hw_ports=(125, 126, 127, 128)),
        _qsfp(label="7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6), hw_ports=(113, 114, 115, 116)),
        _qsfp(label="8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7), hw_ports=(117, 118, 119, 120)),
        _qsfp(label="9", adj_label=None,
              phy_addrs=( 8,  8,  8,  8), hw_ports=( 89,  90,  91,  92)),
        _qsfp(label="10", adj_label=None,
              phy_addrs=( 9,  9,  9,  9), hw_ports=( 93,  94,  95,  96)),
        _qsfp(label="11", adj_label=None,
              phy_addrs=(10, 10, 10, 10), hw_ports=( 81,  82,  83,  84)),
        _qsfp(label="12", adj_label=None,
              phy_addrs=(11, 11, 11, 11), hw_ports=( 85,  86,  87,  88)),
        _qsfp(label="13", adj_label=None,
              phy_addrs=(12, 12, 12, 12), hw_ports=( 73,  74,  75,  76)),
        _qsfp(label="14", adj_label=None,
              phy_addrs=(13, 13, 13, 13), hw_ports=( 77,  78,  79,  80)),
        _qsfp(label="15", adj_label=None,
              phy_addrs=(14, 14, 14, 14), hw_ports=( 65,  66,  67,  68)),
        _qsfp(label="16", adj_label=None,
              phy_addrs=(15, 15, 15, 15), hw_ports=( 69,  70,  71,  72)),
        _qsfp(label="17", adj_label=None,
              phy_addrs=(16, 16, 16, 16), hw_ports=(  5,   6,   7,   8)),
        _qsfp(label="18", adj_label=None,
              phy_addrs=(17, 17, 17, 17), hw_ports=(  1,   2,   3,   4)),
        _qsfp(label="19", adj_label=None,
              phy_addrs=(18, 18, 18, 18), hw_ports=( 13,  14,  15,  16)),
        _qsfp(label="20", adj_label=None,
              phy_addrs=(19, 19, 19, 19), hw_ports=(  9,  10,  11,  12)),
        _qsfp(label="21", adj_label=None,
              phy_addrs=(20, 20, 20, 20), hw_ports=( 21,  22,  23,  24)),
        _qsfp(label="22", adj_label=None,
              phy_addrs=(21, 21, 21, 21), hw_ports=( 17,  18,  19,  20)),
        _qsfp(label="23", adj_label=None,
              phy_addrs=(22, 22, 22, 22), hw_ports=( 29,  30,  31,  32)),
        _qsfp(label="24", adj_label=None,
              phy_addrs=(23, 23, 23, 23), hw_ports=( 25,  26,  27,  28)),
        _qsfpdd(label="25", adj_label=None,
                phy_addrs=(24, 24, 24, 24, 24, 24, 24, 24), hw_ports=( 53,  54,  55,  56)),
        _qsfpdd(label="26", adj_label=None,
                phy_addrs=(25, 25, 25, 25, 25, 25, 25, 25), hw_ports=( 49,  50,  51,  52)),
        _qsfpdd(label="27", adj_label=None,
                phy_addrs=(26, 26, 26, 26, 26, 26, 26, 26), hw_ports=( 61,  62,  63,  64)),
        _qsfpdd(label="28", adj_label=None,
                phy_addrs=(27, 27, 27, 27, 27, 27, 27, 27), hw_ports=( 57,  58,  59,  60)),
        _qsfpdd(label="29", adj_label=None,
                phy_addrs=(28, 28, 28, 28, 28, 28, 28, 28), hw_ports=( 37,  38,  39,  40)),
        _qsfpdd(label="30", adj_label=None,
                phy_addrs=(29, 29, 29, 29, 29, 29, 29, 29), hw_ports=( 33,  34,  35,  36)),
        _qsfpdd(label="31", adj_label=None,
                phy_addrs=(30, 30, 30, 30, 30, 30, 30, 30), hw_ports=( 45,  46,  47,  48)),
        _qsfpdd(label="32", adj_label=None,
                phy_addrs=(31, 31, 31, 31, 31, 31, 31, 31), hw_ports=( 41,  42,  43,  44)),
    )

class MlxSN4410LEDS():
    units = (
        cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxStatusLED(),
        cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxPsuLED(),
        MlxSN4410Fan1LED(),
        MlxSN4410Fan2LED(),
        MlxSN4410Fan3LED(),
        MlxSN4410Fan4LED(),
        MlxSN4410Fan5LED(),
        MlxSN4410Fan6LED(),
    )

class MlxSN4410Sensors():

    def __init__(self):
        self.units = (cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_1(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_2(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_3(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_4(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_5(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_6(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_7(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_8(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_9(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_10(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_11(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_12(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxPSUFan_1(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxPSUFan_2(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_1(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_2(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_3(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_4(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_5(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_6(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_7(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_8(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxPSUTemp_1(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxPSUTemp_2(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxPSU1(),
                      cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxPSU2(),
                      )

class MlxSN4410(cumulus.platform.Platform):
    name = 'mlnx,msn4410'
    chip = cumulus.platform.SpectrumChip()
    switch = MlxSN4410Switch(chip)
    cpld = cumulus.platforms.mlnx_SN4xxx.Mlx4xxxCPLD()
    sensors = MlxSN4410Sensors()
    pwms = None
    statusleds = MlxSN4410LEDS
    use_rate_api = True


Platform = MlxSN4410
