#
# Copyright (C) 2020-2021 NVIDIA Corporation. ALL RIGHTS RESERVED.
# Copyright 2020.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.sensors
import cumulus.sfp


class MlxSimxSN5700QSFP56(cumulus.platform.SpectrumThree400GPort):
    _set_led = '''
;;; MlxSimxSN5700QSFP28
;;; Two bits per port: yellow(active high), green(active high)
'''

    def __init__(self, label, adj_label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.SpectrumThree400GPort.__init__(
            self,
            label=label,
            adj_label=adj_label,
            hw_ports=tuple(hw_port_names),
            phy_addrs=phy_addrs,
            blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp56')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls


class MlxSimxSN5700Switch(cumulus.platform.SpectrumSwitch):

    _sfp = None
    _qsfp = MlxSimxSN5700QSFP56

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can
    #    be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #

    # TODO: Fix SERDES lane mapping as per 5700 HW spec
    ports = (
    )


class MlxSimxSN5700(cumulus.platform.Platform):
    name = 'mlnx,simx_msn5700'
    chip = cumulus.platform.SpectrumChip()
    switch = MlxSimxSN5700Switch(chip)
    pwms = None
    use_rate_api = True
    # SimX platforms do not simulate CPLD/LED/SFP/Fans/Sensors


Platform = MlxSimxSN5700
