#
# Copyright (C) 2020-2021 NVIDIA Corporation. ALL RIGHTS RESERVED.
# Copyright 2016.  Cumulus Networks, Inc.
#

import os
import cumulus.platform
import cumulus.cpld
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.platforms.mlnx_SN2xxx

# System LEDs
class MlxSN2100StatusLED(cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxLED):
    name = 'System'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4',
               'PSU1', 'PSU2',
               'Temp1', 'Temp2', 'Temp3', 'Temp4',
               'Temp5', 'Temp6', 'Temp7']
    led_location = ['led_status_green', 'led_status_red']

class MlxSN2100Psu1LED(cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxLED):
    name = 'Psu1'
    sources = ['PSU1']
    led_location = ['led_psu1_green', 'led_psu1_red']

class MlxSN2100Psu2LED(cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxLED):
    name = 'Psu2'
    sources = ['PSU2']
    led_location = ['led_psu2_green', 'led_psu2_red']

class MlxSN2100FanLED(cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxLED):
    name = 'Fan'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4']
    led_location = ['led_fan_green', 'led_fan_red']

# Base class for the fans
class MlxSN2100Fan(cumulus.sensors.Fan_Unit):
    fan_max = 28000
    fan_min = 700

    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

# System fan #1
class MlxSN2100Fan_1(MlxSN2100Fan):
    name = 'Fan1'
    description = 'Fan 1'
    fan_num = name.lower()
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #2
class MlxSN2100Fan_2(MlxSN2100Fan):
    name = 'Fan2'
    description = 'Fan 2'
    fan_num = name.lower()
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #3
class MlxSN2100Fan_3(MlxSN2100Fan):
    name = 'Fan3'
    description = 'Fan 3'
    fan_num = name.lower()
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #4
class MlxSN2100Fan_4(MlxSN2100Fan):
    name = 'Fan4'
    description = 'Fan 4'
    fan_num = name.lower()
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# Base class for the Temp Sensors
class MlxSN2100Temp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 80
    temp_max = 100
    temp_crit = 115

    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

class MlxSN2100AmbientTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 40
    temp_max = 60
    temp_crit = 70

    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

class MlxSN2100CoreTemp(MlxSN2100Temp):
    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

# System temp sensor #2
class MlxSN2100Temp_2(MlxSN2100Temp):
    name = 'Temp1'
    temp_max = 105
    temp_crit = 120
    description = 'ASIC Temperature Sensor'
    description = 'Asic Temp Sensor'
    mlnx_hw_mgmt_temp_val = 'asic'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = None
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #3
class MlxSN2100Temp_3(MlxSN2100AmbientTemp):
    name = 'Temp2'
    temp_fan_min = 40
    temp_max = 60
    temp_crit = 70
    description = 'Port Ambient Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'port_amb'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = None
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #4
class MlxSN2100Temp_4(MlxSN2100AmbientTemp):
    name = 'Temp3'
    temp_fan_min = 40
    temp_max = 60
    temp_crit = 70
    description = 'Main Board Ambient Sensor'
    mlnx_hw_mgmt_temp_val = 'fan_amb'
    mlnx_hw_mgmt_temp_crit = None
    mlnx_hw_mgmt_temp_max = None
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #5
class MlxSN2100Temp_5(MlxSN2100CoreTemp):
    name = 'Temp4'
    description = 'core0 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core0'
    mlnx_hw_mgmt_temp_crit = 'cpu_core0_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core0_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #6
class MlxSN2100Temp_6(MlxSN2100CoreTemp):
    name = 'Temp5'
    description = 'core1 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core1'
    mlnx_hw_mgmt_temp_crit = 'cpu_core1_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core1_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp senosr #7
class MlxSN2100Temp_7(MlxSN2100CoreTemp):
    name = 'Temp6'
    description = 'core2 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core2'
    mlnx_hw_mgmt_temp_crit = 'cpu_core2_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core2_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #8
class MlxSN2100Temp_8(MlxSN2100CoreTemp):
    name = 'Temp7'
    description = 'core3 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core3'
    mlnx_hw_mgmt_temp_crit = 'cpu_core3_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core3_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# PSU #1
class MlxSN2100PSU1(cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxPSU):
    name = 'PSU1'
    mlnx_hw_mgmt_psu_status = None
    mlnx_hw_mgmt_psu_pwr_status = 'psu1_pwr_status'

# PSU #2
class MlxSN2100PSU2(cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxPSU):
    name = 'PSU2'
    mlnx_hw_mgmt_psu_status = None
    mlnx_hw_mgmt_psu_pwr_status = 'psu2_pwr_status'

class MlxSN2100LEDs():
    units = (
        MlxSN2100StatusLED(),
        MlxSN2100Psu1LED(),
        MlxSN2100Psu2LED(),
        MlxSN2100FanLED(),
    )

class MlxSN2100Sensors():

    def __init__(self):
        self.units = (
                 MlxSN2100Fan_1(),
                 MlxSN2100Fan_2(),
                 MlxSN2100Fan_3(),
                 MlxSN2100Fan_4(),
                 MlxSN2100Temp_2(),
                 MlxSN2100Temp_3(),
                 MlxSN2100Temp_4(),
                 MlxSN2100Temp_5(),
                 MlxSN2100Temp_6(),
                 MlxSN2100Temp_7(),
                 MlxSN2100Temp_8(),
                 MlxSN2100PSU1(),
                 MlxSN2100PSU2(),
        )

class MlxSN2100QSFP28(cumulus.platform.Spectrum100GPort):
    _set_led = '''
;;; MlxSN2100QSFP28
;;;  port LEDs controlled by HW
'''
    def __init__(self, label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.Spectrum100GPort.__init__(self, label=label,
                        adj_label=None, hw_ports=tuple(hw_port_names),
                        phy_addrs=phy_addrs, blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class MlxSN2100Switch(cumulus.platform.SpectrumSwitch):
    _sfp = None
    _qsfp = MlxSN2100QSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        _qsfp(label="1",  phy_addrs=(0,0,0,0),     hw_ports=(25,26,27,28)),
        _qsfp(label="2",  phy_addrs=(1,1,1,1),     hw_ports=(29,30,31,32)),
        _qsfp(label="3",  phy_addrs=(2,2,2,2),     hw_ports=(17,18,19,20)),
        _qsfp(label="4",  phy_addrs=(3,3,3,3),     hw_ports=(21,22,23,24)),
        _qsfp(label="5",  phy_addrs=(4,4,4,4),     hw_ports=( 9,10,11,12)),
        _qsfp(label="6",  phy_addrs=(5,5,5,5),     hw_ports=(13,14,15,16)),
        _qsfp(label="7",  phy_addrs=(6,6,6,6),     hw_ports=(1,2,3,4)),
        _qsfp(label="8",  phy_addrs=(7,7,7,7),     hw_ports=(5,6,7,8)),
        _qsfp(label="9",  phy_addrs=(8,8,8,8),     hw_ports=(37,38,39,40)),
        _qsfp(label="10", phy_addrs=(9,9,9,9),     hw_ports=(33,34,35,36)),
        _qsfp(label="11", phy_addrs=(10,10,10,10), hw_ports=(45,46,47,48)),
        _qsfp(label="12", phy_addrs=(11,11,11,11), hw_ports=(41,42,43,44)),
        _qsfp(label="13", phy_addrs=(12,12,12,12), hw_ports=(53,54,55,56)),
        _qsfp(label="14", phy_addrs=(13,13,13,13), hw_ports=(49,50,51,52)),
        _qsfp(label="15", phy_addrs=(14,14,14,14), hw_ports=(61,62,63,64)),
        _qsfp(label="16", phy_addrs=(15,15,15,15), hw_ports=(57,58,59,60)),
    )

class MlxSN2100(cumulus.platform.Platform):
    name = 'mlnx,x86_MSN2100'
    chip = cumulus.platform.SpectrumChip()
    switch = MlxSN2100Switch(cumulus.platform.SpectrumChip())
    cpld = cumulus.platforms.mlnx_SN2xxx.Mlx2xxxCPLD()
    sensors = MlxSN2100Sensors()
    pwms = None
    statusleds = MlxSN2100LEDs()

Platform = MlxSN2100
