#!/usr/bin/python
# Copyright (C) 2020-2021 NVIDIA Corporation. ALL RIGHTS RESERVED.
# Copyright 2020 Cumulus Networks, Inc.  All rights reserved.
#
# port-modes --
#
#     Return dictionary of where
#       key is port number
#       value is list of ports modes the port can support
#

import os
import sys
import argparse
import json

import cumulus.portvalidator

class ArgParseError(RuntimeError):
    pass

# ----------------------------------------------------------
#
#                         m a i n
#
# ----------------------------------------------------------
def main(argv) :

    parser = argparse.ArgumentParser(
        description='portmodes.py')

    parser.add_argument('-f', '--file',
                        dest="file",
                        required=False,
                        action='store',
                        help='Supply alternate path for ports.conf.',
                        default="/etc/cumulus/ports.conf")

    parser.add_argument('-d', '--debug',
                        required=False,
                        action='store_true',
                        help=argparse.SUPPRESS,
                        default=False)

    try:
        args = parser.parse_args()
    except ArgParseError, e:
        parser.error(str(e))
        exit(-1)

    pv = cumulus.portvalidator.Portvalidator(args.file, args.debug)
    retdict = pv.portmodes()
    print json.dumps(retdict, sort_keys=False, indent=3)
    exit(0)

# ----------------------------------------------------------
#
#                         e n t r y
#
# ----------------------------------------------------------
if __name__ == "__main__":
    main(sys.argv[1:])


