#! /usr/bin/python
#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation and its affiliates
# (the "Company") and all right, title, and interest in and to the software
# product, including all associated intellectual property rights, are and
# shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.

'''
Mellanox PSU EEPROM will not necessarily confirm to IPMI
format; Mellanox BSP team provides the drivers and exports
the EEPROM contents in text file; This is placed under
/run/hw-management/eeprom/psuXX_vpd
'''

class eeprom_psu(object):
    '''

    This class is mainly to support 'decode-syseeprom -t psu'
    command; The methods implemented here will be used to feed
    data to 'decode_eeprom'
    '''
    def __init__(self, name, path, readonly, dummy):
        self.name = name
        self.path = path
        self.readonly = readonly
        self.dummy = dummy
        self.res = {}

    def check_status(self):
        '''
        Check if the PSU is present/absent
        '''
        if self.path <> '':
            file_handle = open(self.path, "r")
            device_status = file_handle.readline().rstrip()
            file_handle.close()
            return device_status
        return 'ok'

    def read_eeprom(self):
        '''
        Open the /run/hw-management/eeprom/psuxx_vpd file to
        read eeprom contents
        '''
        path = str(self.path)
        res = {}
        with open(path) as file_handle:
            for line in file_handle:
                (key, val) = line.strip().split(":")
                res[key] = val

        self.res = res
        return res

    def decode_eeprom(self, res):
        '''
        This method just prints the contents of EEPROM
        '''
        self.res = res
        for key, value in res.items():
            print('{0} \t: {1}'.format(key, value))


    def decode_eeprom_dictionary(self, res):
        '''
        Return the EEPROM contents in a dictionary format
        '''
        return self.res


    def is_checksum_valid(self, arg):
        '''
        Return valid check sum to satisfy the decode-syseeprom
        '''
        return (True, 0)
