#!/bin/sh

#
# Copyright © 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation and its affiliates
# (the "Company") and all right, title, and interest in and to the software
# product, including all associated intellectual property rights, are and
# shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.

[ "$1" = "start" ] || exit 0

# Load the VERBOSE setting and other rcS variables
. /lib/init/vars.sh

# Define LSB log_* functions.
# Depend on lsb-base (>= 3.2-14) to ensure that this file is present
# and status_of_proc is working.
. /lib/lsb/init-functions

# apply sysctl to refresh base_reachable timer values
# arg $1: interface to refresh
refresh_sysctl_base_reachable_timers() {
    iface="$1"
    ipv4_base_reachable_time_ms=$(grep "ipv4.neigh.default.base_reachable_time_ms" /etc/sysctl.d/neigh.conf | cut -d"=" -f2)
    ipv6_base_reachable_time_ms=$(grep "ipv6.neigh.default.base_reachable_time_ms" /etc/sysctl.d/neigh.conf | cut -d"=" -f2)

    if [ ! -z "$ipv4_base_reachable_time_ms" ]; then
        sysctl net.ipv4.neigh.$iface.base_reachable_time_ms=$ipv4_base_reachable_time_ms
    fi

    if [ ! -z "$ipv6_base_reachable_time_ms" ]; then
        sysctl net.ipv6.neigh.$iface.base_reachable_time_ms="$ipv6_base_reachable_time_ms"
    fi
}

log_action_msg "   Refreshing base reachable timers on swp interfaces"
interfaces=$(ip -br link show | grep swp | cut -d" " -f1)
for i in $interfaces ; do
    # base_reachable neighbor sysctl needs to be re-applied to each interface
    # as the sysctl-service has already run at this point.
    refresh_sysctl_base_reachable_timers "$i"
done

