#! /usr/bin/python
# Copyright (C) 2020-2021 NVIDIA Corporation. ALL RIGHTS RESERVED.
# Copyright 2020 Cumulus Networks, Inc.
# All rights reserved

#############################################################################
# Platform and model specific eeprom subclass, inherits from the base class,
# and provides the followings:
# - the eeprom format definition
# - specific encoder/decoder if there is special need
#

try:
    import exceptions
    import eeprom_base
    import eeprom_tlvinfo
    import eeprom_ipmilib_fru as eeprom_ipmi_fru
except ImportError, e:
    raise ImportError (str(e) + "- required module not found")

#
# base board
#
class board(eeprom_tlvinfo.TlvInfoDecoder):

    EEPROM_MAX_SIZE = 0x400
    EEPROM_OFFSET = 0x0

    def __init__(self, name, path, cpld_root, ro):
        super(board, self).__init__(path,
                                    self.EEPROM_OFFSET,
                                    "",
                                    ro,
                                    self.EEPROM_MAX_SIZE)

    def switchaddrstr(self, e):
        return self.base_mac_addr(e)

#
# power supply unit
#
class psu(eeprom_ipmi_fru.IpmiFruDecoder):

    def check_status(self):
        try:
            path = os.path.join('/run/hw-management/thermal/psu%s_pwr_status' % self.name[3])
            with open(path, "r") as fp:
                status = fp.readline().rstrip()
            if int(status):
                return 'ok'
        except:
            pass
        return 'absent'

#
# Fan eeprom
#
class fan(eeprom_base.eepromDecoder):
    fan_fmt = [ ('burn', 'h', 8),
               ('brand', 's', 4),
               ('burn', 'h', 28),
               ('part_num', 's', 12),
               ('checksum', 'c', 2)
               ]

    def __init__(self, name, path, cpld_root, ro):
        self.name = name
        super(fan, self).__init__(path,
                                  self.fan_fmt,
                                  0,
                                  cpld_root,
                                  ro)

    def decoder(self, s, t):
        if s == 'header':
            i = "0x" + binascii.b2a_hex(t).upper()
        else:
            i = t
        return i

    def check_status(self):
        path = '/run/hw-management/thermal/fan%s_status' % self.name[3]
        try:
            with open(path, "r") as fp:
                status = fp.readline().strip()
                if status == '1':
                    return 'ok'
        except:
            pass
        return 'absent'

    def is_checksum_valid(self, e):
        return (True, 0)

